from unittest import TestCase
from math import floor

from benefactor.lists import all_unique, transpose, deep_flatten, difference, difference_by, has_duplicates, \
    to_dictionary, most_frequent, spread


class TestLists(TestCase):
    def test_all_unique(self):
        x = [1, 1, 2, 2, 3, 2, 3, 4, 5, 6]
        y = [1, 2, 3, 4, 5]
        self.assertFalse(all_unique(x))
        self.assertTrue(all_unique(y))

    def test_transpose(self):
        a = [['a', 'b'], ['c', 'd'], ['e', 'f']]
        self.assertEqual(transpose(a), [('a', 'c', 'e'), ('b', 'd', 'f')])

    def test_deep_flatten(self):
        self.assertEqual(deep_flatten([1, [2], [[3], 4], 5]), [1, 2, 3, 4, 5])

    def test_difference(self):
        self.assertEqual(difference([1, 2, 3], [1, 2, 4]), [3])

    def test_difference_by(self):
        self.assertEqual(difference_by([2.1, 1.2], [2.3, 3.4], floor), [1.2])
        self.assertEqual(difference_by([{'x': 2}, {'x': 1}], [{'x': 1}], lambda v: v['x']), [{'x': 2}])

    def test_has_duplicates(self):
        x = [1, 2, 3, 4, 5, 5]
        y = [1, 2, 3, 4, 5]
        self.assertTrue(has_duplicates(x))
        self.assertFalse(has_duplicates(y))

    def test_to_dictionary(self):
        keys = ["a", "b", "c"]
        values = [2, 3, 4]
        self.assertEqual(to_dictionary(keys, values), {'a': 2, 'c': 4, 'b': 3})

    def test_most_frequent(self):
        numbers = [1, 2, 1, 2, 3, 2, 1, 4, 2]
        self.assertEqual(most_frequent(numbers), 2)

    def test_spread(self):
        self.assertEqual(spread([1, 2, 3, [4, 5, 6], [7], 8, 9]), [1, 2, 3, 4, 5, 6, 7, 8, 9])



from collections import Counter


def anagram(first, second):
    return Counter(first) == Counter(second)


def byte_size(string):
    return len(string.encode('utf-8'))


def get_vowels(string):
    return [each for each in string if each in 'aeiou']


def decapitalize(string):
    return string[:1].lower() + string[1:]


def palindrome(a):
    return a == a[::-1]



def all_unique(lst):
    return len(lst) == len(set(lst))


def chunk(lst, size):
    return [lst[i:i + size] for i in range(0, len(lst), size)]


def compact(lst):
    return list(filter(None, lst))


def transpose(lst):
    transposed = zip(*lst)
    return list(transposed)


def spread(arg):
    ret = []
    for i in arg:
        if isinstance(i, list):
            ret.extend(i)
        else:
            ret.append(i)
    return ret


def deep_flatten(xs):
    flat_list = []
    [flat_list.extend(deep_flatten(x)) for x in xs] if isinstance(xs, list) else flat_list.append(xs)
    return flat_list


def difference(a, b):
    set_a = set(a)
    set_b = set(b)
    comparison = set_a.difference(set_b)
    return list(comparison)


def difference_by(a, b, fn):
    b = set(map(fn, b))
    return [item for item in a if fn(item) not in b]


def has_duplicates(lst):
    return len(lst) != len(set(lst))


def to_dictionary(keys, values):
    return dict(zip(keys, values))


def most_frequent(list):
    return max(set(list), key=list.count)



from unittest import TestCase

from benefactor.dicts import merge_two_dicts


class TestDicts(TestCase):
    def test_merge_two_dicts(self):
        a = {'x': 1, 'y': 2}
        b = {'y': 3, 'z': 4}
        self.assertEqual(merge_two_dicts(a, b), {'y': 3, 'x': 1, 'z': 4})



from unittest import TestCase

from benefactor.strings import anagram, byte_size, get_vowels, decapitalize, palindrome


class TestStrings(TestCase):
    def test_anagram(self):
        self.assertTrue(anagram("abcd3", "3acdb"))

    def test_byte_size(self):
        self.assertEqual(byte_size('Hello World'), 11)

    def test_get_vowels(self):
        self.assertEqual(get_vowels('foobar'), ['o', 'o', 'a'])
        self.assertEqual(get_vowels('gym'), [])

    def test_decapitalize(self):
        self.assertEqual(decapitalize("HelloWorld"), "helloWorld")

    def test_palindrome(self):
        self.assertTrue(palindrome('mom'))
        self.assertFalse(palindrome('monday'))
